/* GTK - The GIMP Toolkit
 * Copyright (C) 2000 Red Hat Software
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Owen Taylor <otaylor@redhat.com>
 *         Dinesh Nadarajah <n_dinesh@yahoo.com>
 *
 */

#include <config.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include <gtk/gtkintl.h>
//#include "gtkintl.h"
#include <gtk/gtkimcontextsimple.h>
#include <gtk/gtkimmodule.h>

GType type_tamilvp_uni = 0;

static void tamilvp_uni_class_init (GtkIMContextSimpleClass *class);
static void tamilvp_uni_init (GtkIMContextSimple *im_context);

static void
tamilvp_uni_register_type (GTypeModule *module)
{
  static const GTypeInfo object_info =
  {
    sizeof (GtkIMContextSimpleClass),
    (GBaseInitFunc) NULL,
    (GBaseFinalizeFunc) NULL,
    (GClassInitFunc) tamilvp_uni_class_init,
    NULL,           /* class_finalize */
    NULL,           /* class_data */
    sizeof (GtkIMContextSimple),
    0,
    (GInstanceInitFunc) tamilvp_uni_init,
  };

  type_tamilvp_uni = 
    g_type_module_register_type (module,
				 GTK_TYPE_IM_CONTEXT_SIMPLE,
				 "GtkIMContextTamilVPUni",
				 &object_info, 0);
}

#define SYL1(a,b,c,d,e,f)  a+128, b, c, d, e, f
#define SYL2(a,b,c,d,e,f)  a+128, b+128, c, d, e, f
#define SYL3(a,b,c,d,e,f)  a+128, b+128, c+128, d, e, f
#define SYL4(a,b,c,d,e,f)  a+128, b+128, c+128, d+128, e, f

/* The sequences here match the sequences used in the emacs quail
 * mode cryllic-translit; they allow entering all characters
 * in iso-8859-5
 */
static guint16 tamil_compose_seqs[] = {

  SYL1(GDK_0,	0,		0,		0,		0,		0x30),
  SYL1(GDK_1,	0,		0,		0,		0,		0x0be7),
  SYL1(GDK_2,	0,		0,		0,		0,		0x0be8),
  SYL1(GDK_3,	0,		0,		0,		0,		0x0be9),
  SYL1(GDK_4,	0,		0,		0,		0,		0x0bea),
  SYL1(GDK_5,	0,		0,		0,		0,		0x0beb),
  SYL1(GDK_6,	0,		0,		0,		0,		0x0bec),
  SYL1(GDK_7,	0,		0,		0,		0,		0x0bed),
  SYL1(GDK_8,	0,		0,		0,		0,		0x0bee),
  SYL1(GDK_9,	0,		0,		0,		0,		0x0bef),

  SYL1(GDK_A,	0,		0,		0,		0,		0x0b85),
  SYL2(GDK_A,	GDK_A,	0,		0,		0,		0x0b86),
  SYL2(GDK_A,	GDK_I,	0,		0,		0,		0x0b90),
  SYL2(GDK_A,	GDK_U,	0,		0,		0,		0x0b94),
  SYL1(GDK_B,	0,		0,		0,		0,		0x0000),

  SYL1(GDK_C,	0,		0,		0,		0,		0x0b9e),
  SYL1(GDK_D,	0,		0,		0,		0,		0x0000),

  SYL1(GDK_E,	0,		0,		0,		0,		0x0b8e),
  SYL2(GDK_E,	GDK_E,	0,		0,		0,		0x0b8f),
  SYL1(GDK_F,	0,		0,		0,		0,		0x0000),
  SYL1(GDK_G,	0,		0,		0,		0,		0x0000),
  SYL1(GDK_H,	0,		0,		0,		0,		0x0000),
  SYL1(GDK_I,	0,		0,		0,		0,		0x0b87),
  SYL2(GDK_I,	GDK_I,	0,		0,		0,		0x0b88),

  SYL1(GDK_J,	0,		0,		0,		0,		0x0000),
  SYL1(GDK_K,	0,		0,		0,		0,		0x0000),
  SYL1(GDK_L,	0,		0,		0,		0,		0x0bb3),

  SYL1(GDK_M,	0,		0,		0,		0,		0x0000),
  SYL1(GDK_N,	0,		0,		0,		0,		0x0ba3),

  SYL1(GDK_O,	0,		0,		0,		0,		0x0b92),
  SYL2(GDK_O,	GDK_O,	0,		0,		0,		0x0b93),
  SYL1(GDK_P,	0,		0,		0,		0,		0x0000),
  SYL1(GDK_Q,	0,		0,		0,		0,		0x0b83),

  SYL1(GDK_R,	0,		0,		0,		0,		0x0bb1),

  SYL1(GDK_S,	0,		0,		0,		0,		0x0bb7),
  SYL1(GDK_T,	0,		0,		0,		0,		0x0000),
  SYL1(GDK_U,	0,		0,		0,		0,		0x0b89),
  SYL2(GDK_U,	GDK_U,	0,		0,		0,		0x0b8a),
  SYL1(GDK_V,	0,		0,		0,		0,		0x0000),
  SYL1(GDK_W,	0,		0,		0,		0,		0x0b90),

  SYL1(GDK_X,	0,		0,		0,		0,		0x0000),
  SYL1(GDK_Y,	0,		0,		0,		0,		0x0000),
  SYL1(GDK_Z,	0,		0,		0,		0,		0x0000),

  SYL1(GDK_a,	0,		0,		0,		0,		0x0bbe),
  SYL2(GDK_a,	GDK_i,	0,		0,		0,		0x0bc8),
  SYL2(GDK_a,	GDK_u,	0,		0,		0,		0x0bcc),

  SYL1(GDK_b,	0,		0,		0,		0,		0x0ba8),

  SYL1(GDK_c,	0,		0,		0,		0,		0x0b9a),

  SYL1(GDK_d,	0,		0,		0,		0,		0x0ba4),

  SYL1(GDK_e,	0,		0,		0,		0,		0x0bc6),
  SYL2(GDK_e,	GDK_e,	0,		0,		0,		0x0bc7),

  SYL1(GDK_f,	0,		0,		0,		0,		0x0000),

  SYL1(GDK_g,	0,		0,		0,		0,		0x0b99),

  SYL1(GDK_h,	0,		0,		0,		0,		0x0bb9),

  SYL1(GDK_i,	0,		0,		0,		0,		0x0bbf),
  SYL2(GDK_i,	GDK_i,	0,		0,		0,		0x0bc0),

  SYL1(GDK_j,	0,		0,		0,		0,		0x0b9c),

  SYL1(GDK_k,	0,		0,		0,		0,		0x0b95),

  SYL1(GDK_l,	0,		0,		0,		0,		0x0bb2),

  SYL1(GDK_m,	0,		0,		0,		0,		0x0bae),

  SYL1(GDK_n,	0,		0,		0,		0,		0x0ba9),
  SYL2(GDK_n,	GDK_g,	0,		0,		0,		0x0b99),
  SYL2(GDK_n,	GDK_j,	0,		0,		0,		0x0b9e),

  SYL1(GDK_o,	0,		0,		0,		0,		0x0bca),
  SYL2(GDK_o,	GDK_o,	0,		0,		0,		0x0bcb),

  SYL1(GDK_p,	0,		0,		0,		0,		0x0baa),

  SYL1(GDK_q,	0,		0,		0,		0,		0x0bcd),

  SYL1(GDK_r,	0,		0,		0,		0,		0x0bb0),

  SYL1(GDK_s,	0,		0,		0,		0,		0x0bb8),
  SYL2(GDK_s,	GDK_h,	0,		0,		0,		0x0bb7),  

  SYL1(GDK_t,	0,		0,		0,		0,		0x0b9f),
  SYL2(GDK_t,	GDK_h,	0,		0,		0,		0x0ba4),

  SYL1(GDK_u,	0,		0,		0,		0,		0x0bc1),
  SYL2(GDK_u,	GDK_u,	0,		0,		0,		0x0bc2),

  SYL1(GDK_v,	0,		0,		0,		0,		0x0bb5),

  SYL1(GDK_w,	0,		0,		0,		0,		0x0bc8),

  SYL1(GDK_x,	0,		0,		0,		0,		0x0000),

  SYL1(GDK_y,	0,		0,		0,		0,		0x0baf),

  SYL1(GDK_z,	0,		0,		0,		0,		0x0bb4)

};



static void
tamilvp_uni_class_init (GtkIMContextSimpleClass *class)
{
}

static void
tamilvp_uni_init (GtkIMContextSimple *im_context)
{
  gtk_im_context_simple_add_table (im_context,
				   tamil_compose_seqs,
				   4,
				   G_N_ELEMENTS (tamil_compose_seqs) / (4 + 2));
}

static const GtkIMContextInfo tamilvp_uni_info = { 
  "tamilvp_uni",		   /* ID */
  N_("TamilVP (UNICODE)"), /* Human readable name */
  "gtk+",			   /* Translation domain */
   "",		   /* Dir for bindtextdomain (not strictly needed for "gtk+") */
  "ta_IN.UTF8"			           /* Languages for which this module is the default */
};

static const GtkIMContextInfo *info_list[] = {
  &tamilvp_uni_info
};

void
im_module_init (GTypeModule *module)
{
  tamilvp_uni_register_type (module);
}

void 
im_module_exit (void)
{
}

void 
im_module_list (const GtkIMContextInfo ***contexts,
		int                      *n_contexts)
{
  *contexts = info_list;
  *n_contexts = G_N_ELEMENTS (info_list);
}

GtkIMContext *im_module_create (const gchar *context_id)
{
  if (strcmp (context_id, "tamilvp_uni") == 0)
    return GTK_IM_CONTEXT (g_object_new (type_tamilvp_uni, NULL));
  else
    return NULL;
}
